package com.gct.common.core.shardingJDBC;

import com.zaxxer.hikari.HikariDataSource;
import org.apache.shardingsphere.api.config.sharding.ShardingRuleConfiguration;
import org.apache.shardingsphere.api.config.sharding.TableRuleConfiguration;
import org.apache.shardingsphere.api.config.sharding.strategy.ComplexShardingStrategyConfiguration;
import org.apache.shardingsphere.shardingjdbc.api.ShardingDataSourceFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.Primary;

import javax.print.attribute.standard.PrinterURI;
import javax.sql.DataSource;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * class name: ShardingJDBCConfiguration
 *
 * @author gxt
 * @version 1.0
 * @since 2021/10/12 下午6:34 周二
 */
@Configuration
@EnableConfigurationProperties({ShardingJDBCProperties.class,DataSourceProperties.class})
/*@ConditionalOnProperty(
        prefix = "spring.gct.sharding",
        name = {"enabled"},
        havingValue = "true",
        matchIfMissing = true
)*/
public class ShardingJDBCConfiguration   {


    public static ThreadLocal<Map<String, String>> threadLocal = new ThreadLocal<>();

    public DataSource createDataSource(String driverClassName,String jdbcUrl,String username,String password) {
        HikariDataSource result = new HikariDataSource();
        result.setDriverClassName(driverClassName);
        result.setJdbcUrl(jdbcUrl);
        result.setUsername(username);
        result.setPassword(password);
        return result;
    }
    @Bean
    @Primary
    public DataSource getShardingDataSource(ShardingJDBCProperties annotation,DataSourceProperties dataSourceProperties) throws SQLException{
        Map<String, DataSource> dataSourceMap = new HashMap<>();
        dataSourceMap.put(annotation.getDsNumValue(),createDataSource(dataSourceProperties.getDriverClassname(), dataSourceProperties.getUrl(),
                dataSourceProperties.getUserName(),dataSourceProperties.getPassWord()));
        Map<String, String> map = new HashMap<>();
        map.put("dataSourceNameValueForEx",annotation.getDataSourceNameValueForEx()+".");
        map.put("cloumnNameValue",annotation.getCloumnNameValue());
        threadLocal.set(map);
// 配置表规则
        TableRuleConfiguration tableRuleConfig = new TableRuleConfiguration(annotation.getTableNameValue(), annotation.getDsNumValue()+"."+annotation.getTableNameValue());
// 配置分库 + 分表策略
        tableRuleConfig.setTableShardingStrategyConfig(new ComplexShardingStrategyConfiguration(annotation.getCloumnNameValue(), new ShardingRuleComplexKeys()));
        tableRuleConfig.setTableShardingStrategyConfig(new ComplexShardingStrategyConfiguration(annotation.getCloumnNameValue(), new ShardingRuleComplexKeysWithStrig()));
// 配置分片规则
        ShardingRuleConfiguration shardingRuleConfig = new ShardingRuleConfiguration();
        shardingRuleConfig.getTableRuleConfigs().add(tableRuleConfig);
        Properties props = new Properties();
        props.setProperty("sql.show","true");
        return ShardingDataSourceFactory.createDataSource(dataSourceMap, shardingRuleConfig, props);
    }

    /*@Bean
    @Primary
    public DataSource dataSource() throws SQLException {
        Map<String, Object> beans =ShardingJDBCConfigurationContextAware.getBeansByAnnotation(ShardingJDBCAnnotation.class);
        System.out.println("bean--->"+beans.size());
        for (Object value : beans.values()) {
            System.out.println("vale---->"+value);
            ShardingJDBCAnnotation annotation = value.getClass().getAnnotation(ShardingJDBCAnnotation.class);
           return getShardingDataSource(annotation);
        }
        return new HikariDataSource();
    }*/


}

