package com.gct.common.core.shardingJDBC;

import com.google.common.collect.Range;
import org.apache.shardingsphere.api.sharding.complex.ComplexKeysShardingAlgorithm;
import org.apache.shardingsphere.api.sharding.complex.ComplexKeysShardingValue;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * class name: ShardingRuleComplexKeys
 * 自定义复合分片 可用作 > < >= <= in == between and </>
 *
 * @author gxt
 * @version 1.0
 * @since 2021/10/12 下午4:41 周二
 */
public class ShardingRuleComplexKeysWithStrig implements ComplexKeysShardingAlgorithm<String> {
    DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    @Override
    public Collection<String> doSharding(Collection<String> collection, ComplexKeysShardingValue<String> complexKeysShardingValue) {
        Collection<String> result = new ArrayList<>();
        Map<String, String> map = ShardingJDBCConfiguration.threadLocal.get();
        if (map.size() > 0) {
            if (!Objects.isNull(complexKeysShardingValue)) {
                if (!StringUtils.isEmpty(complexKeysShardingValue.getColumnNameAndShardingValuesMap().get(map.get("cloumnNameValue")))) {
                    List<String> prod_date = (List<String>) complexKeysShardingValue.getColumnNameAndShardingValuesMap().get(map.get("cloumnNameValue"));
                    String[] date = prod_date.get(0).toString().split("-");
                    result.add(map.get("dataSourceNameValueForEx") + complexKeysShardingValue.getLogicTableName() + "_" + date[0] + date[1]);
                } else if (!StringUtils.isEmpty(complexKeysShardingValue.getColumnNameAndRangeValuesMap().get(map.get("cloumnNameValue")))) {
                    Range<String> prod_date = complexKeysShardingValue.getColumnNameAndRangeValuesMap().get(map.get("cloumnNameValue"));
                    String[] splitStr = prod_date.toString().replace("[", "").replace("]", "").split("\\..");
                    if (prod_date.hasLowerBound()) {
                        String[] split = splitStr[0].split("-");
                        split[0] = split[0].replace("(","");
                        String start = map.get("dataSourceNameValueForEx") + complexKeysShardingValue.getLogicTableName() + "_" + split[0] + split[1];
                        result.add(start);
                    }
                    if (prod_date.hasUpperBound()) {
                        String[] split = splitStr[1].split("-");
                        split[1] = split[1].replace(")","");
                        String end = map.get("dataSourceNameValueForEx") + complexKeysShardingValue.getLogicTableName() + "_" + split[0] + split[1];
                        result.add(end);
                    }
                }
                ShardingJDBCConfiguration.threadLocal.remove();
                if (result.size() > 0) return result;
            }
        }
        return result;
    }
}
