package com.gct.common.security.config;

import com.alibaba.fastjson.support.spring.FastJsonRedisSerializer;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.jedis.JedisClientConfiguration;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.StringRedisSerializer;
import redis.clients.jedis.JedisPoolConfig;

/**
 * class name: RedisConfig
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/5/6 下午5:02
 */
@Configuration
@ConditionalOnMissingBean(GCTRedisCacheConfig.class)
//@EnableCaching
@EnableConfigurationProperties({RedisClientConfigProperties.class,RedisClientJedisPoolProperties.class})
public class GCTRedisCacheConfig {

    @Autowired
//    @Qualifier("spring.redis-com.gct.common.security.config.RedisClientConfigProperties")
    RedisClientConfigProperties redisClientConfigProperties;

    @Autowired
    RedisClientJedisPoolProperties redisClientJedisPoolProperties;
    @Bean
    @ConditionalOnMissingBean(RedisConnectionFactory.class)
    public RedisConnectionFactory redisConnectionFactory() {
        JedisPoolConfig jedisPoolConfig = new JedisPoolConfig();
        jedisPoolConfig.setMaxIdle(redisClientJedisPoolProperties.getMaxIdle());
        jedisPoolConfig.setMinIdle(redisClientJedisPoolProperties.getMinIdle());
        jedisPoolConfig.setMaxWaitMillis(redisClientJedisPoolProperties.getMaxWait());
        jedisPoolConfig.setMaxTotal(redisClientJedisPoolProperties.getMaxTotal());
        JedisClientConfiguration.JedisPoolingClientConfigurationBuilder jedisPoolingClientConfigurationBuilder = (JedisClientConfiguration.JedisPoolingClientConfigurationBuilder) JedisClientConfiguration.builder();
        jedisPoolingClientConfigurationBuilder.poolConfig(jedisPoolConfig);
        LoggerFactory.getLogger(this.getClass()).warn("\033[1;35mLoaded redis config:{},pool config:{}\033[0m ", redisClientConfigProperties.toString(),redisClientJedisPoolProperties.toString());

        RedisStandaloneConfiguration redisStandaloneConfiguration = new RedisStandaloneConfiguration(redisClientConfigProperties.getHost(), redisClientConfigProperties.getPort());
        redisStandaloneConfiguration.setDatabase(redisClientConfigProperties.getDatabase());
        redisStandaloneConfiguration.setPassword(redisClientConfigProperties.getPassword());

        JedisClientConfiguration jedisClientConfiguration = jedisPoolingClientConfigurationBuilder.build();
        return new JedisConnectionFactory(redisStandaloneConfiguration, jedisClientConfiguration);
    }

    @Bean
    @ConditionalOnMissingBean(RedisTemplate.class)
    public RedisTemplate<Object, Object> redisTemplate(RedisConnectionFactory connectionFactory) {
        RedisTemplate<Object, Object> template = new RedisTemplate<>();
        template.setConnectionFactory(connectionFactory);
        //使用FastJsonRedisSerializer来序列化和反序列化redis的value值（默认使用JDK的序列化方式）
        template.setValueSerializer(new FastJsonRedisSerializer<>(Object.class));
        //使用StringRedisSerializer来序列化和反序列化redis的key值
        template.setKeySerializer(new StringRedisSerializer());
        template.afterPropertiesSet();
        return template;
    }
}
