package com.gct.common.security.core;

import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;

import java.time.Duration;
import java.util.Date;

/**
 * class name: DefaultCheckTokenCaching
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/5/8 下午1:54
 */
@Slf4j
public class DefaultCheckTokenCaching implements CheckTokenCaching {
    private RedisTemplate redisTemplate;

    public DefaultCheckTokenCaching(RedisTemplate redisTemplate) {
        this.redisTemplate = redisTemplate;
    }

    /**
     * @param token
     * @return true:exited false:not exited
     */
    @Override
    public boolean isExitedInCache(String token) {
        try {
            Object o = redisTemplate.opsForValue().get(token);
            return o != null;
        } catch (Exception e) {
            log.error("get token from redis failed:{}...,but return true(temporary)",
                    token == null || token.length() < 10 ? token : token.substring(10));
            return true;//redis运行出错，继续使用现有token
        }
    }

    /**
     * @param token
     * @param expirationDate
     * @return
     */
    @Override
    public boolean setTokenToCache(String token, Date expirationDate) {
        try {
            redisTemplate.opsForValue().set(token, 1, Duration.ofMillis(1000));
            redisTemplate.expireAt(token, expirationDate);
            log.info("save token to redis failed(false) or completed(true)?-->true:{}", token);
            return true;
        } catch (Exception e) {
            log.error("save token to redis failed(false) or completed(true)?-->false:{}", token);
//            e.printStackTrace();
            return false;
        }
    }

    @Override
    public boolean deleteTokenFromCache(String token) {
        try {
            redisTemplate.delete(token);
            log.info("delete token from redis successfully:{}", token);
            return true;
        } catch (Exception e) {
            log.error("delete token from redis failed:{}", token);
//            e.printStackTrace();
            return false;
        }
    }
}
