package com.gct.common.util;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 示功图相关工具类
 *
 * @author: NewMeanning
 * @create: 2021-01-26 17:51
 **/
public class SGTUtil {


    private static DecimalFormat df = new DecimalFormat("#0.00");

    /**
     * 解析前三后五类型示功图返回List
     *
     * @param sgt
     * @return
     */
    public static List<Integer> encode35ToList(String sgt) {
        ArrayList<Integer> list = new ArrayList<>();

        if (StringUtil.isBlankOrEmpty(sgt)) return list;
        for (int i = 0; i < sgt.length() / 8; i++) {
            list.add(Integer.valueOf(sgt.substring(i * 8, i * 8 + 3)));
            list.add(Integer.valueOf(sgt.substring(i * 8 + 3, i * 8 + 8)));
        }
        return list;
    }

    /**
     * 解析前三后五类型示功图返回array
     *
     * @param sgt
     * @return
     */
    public static Integer[] encode35Array(String sgt) {
        if (StringUtil.isBlankOrEmpty(sgt)) return new Integer[]{};
        List<Integer> list = encode35ToList(sgt);
        Integer[] array = list.toArray(new Integer[sgt.length() / 8]);
        return array;
    }

    /**
     * 解析前三后五类型示功图返回“，”隔开的字符串
     *
     * @param sgt
     * @return
     */
    public static String encode35String(String sgt) {
        if (StringUtil.isBlankOrEmpty(sgt)) return "";
        return encode35ToList(sgt).stream().map(String::valueOf).collect(Collectors.joining(","));
    }

    /**
     * 位移载荷逗号隔开的转list
     *
     * @param sgt
     * @return
     */
    public static List<Integer> encodeCommaToList(String sgt) {
        List<Integer> list = new ArrayList<>();

        if (StringUtil.isBlankOrEmpty(sgt)) return list;
        String[] split = sgt.split(",");
        for (int i = 0; i < split.length; i++) {
            Long round = Math.round(Double.valueOf(split[i]) * 100);
            list.add(round.intValue());
        }
        return list;
    }

    /**
     * 位移载荷逗号隔开的转array
     *
     * @param sgt
     * @return
     */
    public static Integer[] encodeCommaToArray(String sgt) {
        if (StringUtil.isBlankOrEmpty(sgt)) return new Integer[]{};
        String[] split = sgt.split(",");
        Integer[] array = new Integer[split.length];
        for (int i = 0; i < split.length; i++) {
            array[i] = Integer.valueOf(df.format(split[i]).replace(".", ""));
        }
        return array;
    }

    /**
     * 位移载荷逗号隔开的转String
     *
     * @param sgt
     * @return
     */
    public static String encodeCommaToString(String sgt) {
        if (StringUtil.isBlankOrEmpty(sgt)) return "";
        return encodeCommaToList(sgt).stream().map(String::valueOf).collect(Collectors.joining(","));
    }

    /**
     * 将存储格式功图转换为前端需要的格式
     *
     * @param sgt
     * @return
     */
    public static List<Double[]> encodeToView(String sgt) {
        List<Double[]> list = new ArrayList<>();
        if (StringUtil.isBlankOrEmpty(sgt)) return list;

        String[] split = sgt.split(",");
        for (int i = 0; i < split.length; i++) {
            if (i % 2 == 1) {
                list.add(new Double[]{
                        Double.valueOf(df.format(Double.valueOf(split[i - 1]) / 100)),
                        Double.valueOf(df.format(Double.valueOf(split[i]) / 100))
                });
            }
        }
        return list;
    }

    /**
     * 将存储格式功图转换为程序可用的list
     *
     * @return
     */
    public static List<List<Double>> encodeToUseList(String sgt) {
        List<List<Double>> list = new ArrayList<>();

        if (StringUtil.isBlankOrEmpty(sgt)) return list;
        String[] split = sgt.split(",");
        for (int i = 0; i < split.length; i++) {
            if (i % 2 == 1) {
                List<Double> li = new ArrayList<>();
                li.add(Double.valueOf(df.format(Double.valueOf(split[i - 1]) / 100)));
                li.add(Double.valueOf(df.format(Double.valueOf(split[i]) / 100)));
                list.add(li);
            }
        }
        return list;
    }

    /**
     * 将存储格式功图转换为程序可用的二维数组
     *
     * @param sgt
     * @return
     */
    public static double[][] encodeToUseArray(String sgt) {
        if (StringUtil.isBlankOrEmpty(sgt)) return new double[][]{};
        String[] split = sgt.split(",");
        double[][] array = new double[split.length / 2][2];
        for (int i = 0, j = 0; i < split.length; i++) {
            if (i % 2 == 1) {
                array[j][0] = Double.valueOf(df.format(Double.valueOf(split[i - 1]) / 100));
                array[j][1] = Double.valueOf(df.format(Double.valueOf(split[i]) / 100));
                j++;
            }
        }
        return array;
    }

    /**
     * @param sgt
     * @return
     */
    public static Double[][] encodeToDoubleArray(String sgt) {
        if (StringUtil.isBlankOrEmpty(sgt)) return new Double[][]{};
        String[] split = sgt.split(",");
        if (split.length % 2 != 0) return new Double[][]{};
        int totalPointNum = split.length / 2;
        Double[][] rs = new Double[totalPointNum][];
        for (int i = 0; i < totalPointNum; i++) {
            rs[i] = new Double[]{
                    Double.valueOf(df.format(Double.valueOf(split[i * 2]) / 100)),
                    Double.valueOf(df.format(Double.valueOf(split[i * 2 + 1]) / 100))
            };
        }
        return rs;
    }

    public static Double[][] encodeToDoubleArrayReverse(String sgt) {
        if (StringUtil.isBlankOrEmpty(sgt)) return new Double[][]{};
        String[] split = sgt.split(",");
        if (split.length % 2 != 0) return new Double[][]{};
        int totalPointNum = split.length / 2;
        Double[][] rs = new Double[2][totalPointNum];
        for (int i = 0; i < totalPointNum; i++) {
            rs[0][i] = Double.valueOf(df.format(Double.valueOf(split[i]) / 100));
            rs[1][i] = Double.valueOf(df.format(Double.valueOf(split[i + totalPointNum]) / 100));
        }
        return rs;

    }

    public static String encodeToString(Double[][] gt) {
        if (gt == null || gt.length == 0 || gt[0].length == 0) return "";
        String sgtStr = "";
        for (int i = 0; i < gt.length; i++)
            for (int j = 0; j < gt[i].length; j++)
                if (i == 0 && j == 0) sgtStr += "" + Math.round(gt[i][j] * 100);
                else sgtStr += "," + Math.round(gt[i][j] * 100);
        return sgtStr;

    }

    public static String encodeToString(double[][] gt) {
        if (gt == null || gt.length == 0 || gt[0].length == 0) return "";
        String sgtStr = "";
        for (int i = 0; i < gt.length; i++)
            for (int j = 0; j < gt[i].length; j++)
                if (i == 0 && j == 0) sgtStr += "" + Math.round(gt[i][j] * 100);
                else sgtStr += "," + Math.round(gt[i][j] * 100);
        return sgtStr;

    }

    public static Double[][] diagramReverse(Double[][] gt) {
        if (gt == null || gt.length == 0 || gt[0].length == 0) return new Double[][]{};
        int rows = gt.length;
        int cols = gt[0].length;
        Double[][] rs = new Double[cols][rows];
        for (int i = 0; i < rows; i++)
            for (int j = 0; j < cols; j++)
                rs[j][i] = gt[i][j];
        return rs;
    }

    //测试
    public static void main(String[] args) {
        String sgt = "0,4240,0,4208,2,4240,4,4272,7,4368,11,4432,14,4464,19,4512,25,4624,30,4736,37,4992,43,5120,51,5344,61,5536,68,5664,78,5968,89,6352,97,6528,109,6848,122,7023,130,7023,144,6895,153,6768,167,6528,181,6384,190,6352,205,6400,220,6559,230,6656,245,6800,259,6831,269,6800,284,6704,294,6640,309,6495,323,6416,332,6416,346,6480,360,6559,369,6608,382,6656,390,6656,402,6623,413,6544,421,6495,431,6416,441,6368,448,6368,456,6384,464,6400,469,6416,476,6416,480,6400,485,6336,490,6256,492,6224,495,6160,498,6144,499,6144,499,6176,499,6208,499,6208,497,6176,496,6128,492,6064,489,6032,485,6016,480,5968,474,6000,469,6032,462,6096,456,6112,448,6128,438,6112,432,6096,421,6032,410,5984,402,5984,390,6000,378,6000,369,6000,356,6000,347,5984,333,5776,319,5280,309,4960,294,4272,280,3696,270,3488,255,3232,240,3296,230,3456,215,3904,205,4128,191,4448,176,4480,167,4400,153,4192,140,3872,131,3696,118,3536,109,3584,97,3744,86,3936,78,4064,68,4304,58,4352,52,4320,43,4240,35,4096,30,4015,23,3936,19,3936,14,4047,9,4160,7,4240,4,4384,1,4448,0,4432,0,4336";
//        List<Integer> integers = encode35ToList(sgt);
//        Integer[] array = encode35Array(sgt);
//        String s = encode35String(sgt);
        double[][] doubles = encodeToUseArray(sgt);
        System.out.println(doubles);
    }

}
